@@ -86,7 +86,7 @@ defaults = [
     ('ipv6_enabled', 0,
          'allow the client to connect to peers via IPv6'),
     ('ipv6_binds_v4', autodetect_socket_style(),
-        'set if an IPv6 server socket will also field IPv4 connections'),
+        "set if an IPv6 server socket won't also field IPv4 connections"),
     ('upnp_nat_access', 1,
         'attempt to autoconfigure a UPnP router to forward a server port ' +
         '(0 = disabled, 1 = mode 1 [fast], 2 = mode 2 [slow])'),
@@ -120,6 +120,9 @@ defaults = [
     ('write_buffer_size', 4,
         'the maximum amount of space to use for buffering disk writes ' +
         '(in megabytes, 0 = disabled)'),
+    ('breakup_seed_bitfield', 1,
+        'sends an incomplete bitfield and then fills with have messages, '
+        'in order to get around stupid ISP manipulation'),
     ('snub_time', 30.0,
         "seconds to wait for data to come in over a connection before assuming it's semi-permanently choked"),
     ('spew', 0,
@@ -157,9 +160,13 @@ defaults = [
 argslistheader = 'Arguments are:\n\n'
 
 
+def _failfunc(x):
+    print x
+
 # old-style downloader
-def download(self, params, filefunc, statusfunc, finfunc, errorfunc, doneflag, cols,
-             pathFunc = None, presets = {}, exchandler = None):
+def download(params, filefunc, statusfunc, finfunc, errorfunc, doneflag, cols,
+             pathFunc = None, presets = {}, exchandler = None,
+             failed = _failfunc, paramfunc = None):
 
     try:
         config = parse_params(params, presets)
@@ -251,7 +258,7 @@ def get_usage(defaults = defaults, cols = 100, presets = {}):
     return (argslistheader + formatDefinitions(defaults, cols, presets))
 
 
-def get_response(file, url, failfunc):
+def get_response(file, url, errorfunc):
     try:
         if file:
             h = open(file, 'rb')
@@ -261,7 +268,7 @@ def get_response(file, url, failfunc):
                 assert front[0] == 'd'
                 int(front[1:])
             except:
-                failfunc(file+' is not a valid responsefile')
+                errorfunc(file+' is not a valid responsefile')
                 return None
             try:
                 h.seek(0)
@@ -275,22 +282,26 @@ def get_response(file, url, failfunc):
             try:
                 h = urlopen(url)
             except:
-                failfunc(url+' bad url')
+                errorfunc(url+' bad url')
                 return None
         response = h.read()
     
     except IOError, e:
-        failfunc('problem getting response info - ' + str(e))
+        errorfunc('problem getting response info - ' + str(e))
         return None
     try:    
         h.close()
     except:
         pass
     try:
-        response = bdecode(response)
+        try:
+            response = bdecode(response)
+        except:
+            errorfunc("warning: bad data in responsefile")
+            response = bdecode(response, sloppy=1)
         check_message(response)
     except ValueError, e:
-        failfunc("got bad file info - " + str(e))
+        errorfunc("got bad file info - " + str(e))
         return None
 
     return response
@@ -319,6 +330,7 @@ class BT1Download:
         self.argslistheader = argslistheader
         self.unpauseflag = Event()
         self.unpauseflag.set()
+        self.downloader = None
         self.storagewrapper = None
         self.fileselector = None
         self.super_seeding_active = False
@@ -480,7 +492,9 @@ class BT1Download:
 
             data = self.appdataobj.getTorrentData(self.infohash)
             try:
-                disabled_files = [x == -1 for x in data['resume data']['priority']]
+                d = data['resume data']['priority']
+                assert len(d) == len(self.files)
+                disabled_files = [x == -1 for x in d]
             except:
                 try:
                     disabled_files = [x == -1 for x in self.priority]
@@ -500,7 +514,7 @@ class BT1Download:
             self.storagewrapper = StorageWrapper(self.storage, self.config['download_slice_size'],
                 self.pieces, self.info['piece length'], self._finished, self._failed,
                 statusfunc, self.doneflag, self.config['check_hashes'],
-                self._data_flunked, self.rawserver.external_add_task,
+                self._data_flunked, self.rawserver.add_task,
                 self.config, self.unpauseflag)
             
         except ValueError, e:
@@ -514,7 +528,7 @@ class BT1Download:
             self.fileselector = FileSelector(self.files, self.info['piece length'],
                                              self.appdataobj.getPieceDir(self.infohash),
                                              self.storage, self.storagewrapper,
-                                             self.rawserver.external_add_task,
+                                             self.rawserver.add_task,
                                              self._failed)
             if data:
                 data = data.get('resume data')
@@ -534,8 +548,7 @@ class BT1Download:
     def _make_upload(self, connection, ratelimiter, totalup):
         return Upload(connection, ratelimiter, totalup,
                       self.choker, self.storagewrapper, self.picker,
-                      self.config['max_slice_length'], self.config['max_rate_period'],
-                      self.config['upload_rate_fudge'], self.config['buffer_reads'])
+                      self.config)
 
     def _kick_peer(self, connection):
         def k(connection = connection):
@@ -647,11 +660,12 @@ class BT1Download:
         self.rerequest = Rerequester(trackerlist, self.config['rerequest_interval'], 
             self.rawserver.add_task, self.connecter.how_many_connections, 
             self.config['min_peers'], self.encoder.start_connections,
-            self.rawserver.external_add_task, self.storagewrapper.get_amount_left, 
+            self.rawserver.add_task, self.storagewrapper.get_amount_left, 
             self.upmeasure.get_total, self.downmeasure.get_total, self.port, self.config['ip'],
             self.myid, self.infohash, self.config['http_timeout'],
             self.errorfunc, self.excfunc, self.config['max_initiate'],
-            self.doneflag, self.upmeasure.get_rate, self.downmeasure.get_rate)
+            self.doneflag, self.upmeasure.get_rate, self.downmeasure.get_rate,
+            self.unpauseflag )
 
         self.rerequest.start()
 
@@ -711,7 +725,7 @@ class BT1Download:
             def s(self = self, rate = rate):
                 self.config['max_upload_rate'] = rate
                 self.ratelimiter.set_upload_rate(rate)
-            self.rawserver.external_add_task(s)
+            self.rawserver.add_task(s)
         except AttributeError:
             pass
 
@@ -724,7 +738,7 @@ class BT1Download:
                 self.config['max_uploads'] = conns2
                 if (conns > 30):
                     self.config['max_initiate'] = conns + 10
-            self.rawserver.external_add_task(s)
+            self.rawserver.add_task(s)
         except AttributeError:
             pass
         
@@ -733,21 +747,21 @@ class BT1Download:
             def s(self = self, rate = rate):
                 self.config['max_download_rate'] = rate
                 self.downloader.set_download_rate(rate)
-            self.rawserver.external_add_task(s)
+            self.rawserver.add_task(s)
         except AttributeError:
             pass
 
     def startConnection(self, ip, port, id):
-        self.encrypter._start_connection((ip, port), id)
+        self.encoder._start_connection((ip, port), id)
       
     def _startConnection(self, ipandport, id):
-        self.encrypter._start_connection(ipandport, id)
+        self.encoder._start_connection(ipandport, id)
         
     def setInitiate(self, initiate):
         try:
             def s(self = self, initiate = initiate):
                 self.config['max_initiate'] = initiate
-            self.rawserver.external_add_task(s)
+            self.rawserver.add_task(s)
         except AttributeError:
             pass
 
@@ -767,7 +781,7 @@ class BT1Download:
                     self.rerequest.announce()
                 else:
                     self.rerequest.announce(specialurl = special)
-            self.rawserver.external_add_task (r)
+            self.rawserver.add_task(r)
         except AttributeError:
             pass
 
@@ -777,25 +791,53 @@ class BT1Download:
         except:
             return None
 
+#    def Pause(self):
+#        try:
+#            if self.storagewrapper:
+#                self.rawserver.add_task(self._pausemaker, 0)
+#        except:
+#            return False
+#        self.unpauseflag.clear()
+#        return True
+#
+#    def _pausemaker(self):
+#        self.whenpaused = clock()
+#        self.unpauseflag.wait()   # sticks a monkey wrench in the main thread
+#
+#    def Unpause(self):
+#        self.unpauseflag.set()
+#        if self.whenpaused and clock()-self.whenpaused > 60:
+#            def r(self = self):
+#                self.rerequest.announce(3)      # rerequest automatically if paused for >60 seconds
+#            self.rawserver.add_task(r)
+
     def Pause(self):
-        try:
-            if self.storagewrapper:
-                self.rawserver.external_add_task(self._pausemaker, 0)
-        except:
+        if not self.storagewrapper:
             return False
         self.unpauseflag.clear()
+        self.rawserver.add_task(self.onPause)
         return True
 
-    def _pausemaker(self):
+    def onPause(self):
         self.whenpaused = clock()
-        self.unpauseflag.wait()   # sticks a monkey wrench in the main thread
-
+        if not self.downloader:
+            return
+        self.downloader.pause(True)
+        self.encoder.pause(True)
+        self.choker.pause(True)
+    
     def Unpause(self):
         self.unpauseflag.set()
-        if self.whenpaused and clock()-self.whenpaused > 60:
-            def r(self = self):
-                self.rerequest.announce(3)      # rerequest automatically if paused for >60 seconds
-            self.rawserver.external_add_task(r)
+        self.rawserver.add_task(self.onUnpause)
+
+    def onUnpause(self):
+        if not self.downloader:
+            return
+        self.downloader.pause(False)
+        self.encoder.pause(False)
+        self.choker.pause(False)
+        if self.rerequest and self.whenpaused and clock()-self.whenpaused > 60:
+            self.rerequest.announce(3)      # rerequest automatically if paused for >60 seconds
 
     def set_super_seed(self):
         try:
@@ -803,7 +845,7 @@ class BT1Download:
             def s(self = self):
                 if self.finflag.isSet():
                     self._set_super_seed()
-            self.rawserver.external_add_task(s)
+            self.rawserver.add_task(s)
         except AttributeError:
             pass
 
@@ -815,14 +857,14 @@ class BT1Download:
             def s(self = self):
                 self.downloader.set_super_seed()
                 self.choker.set_super_seed()
-            self.rawserver.external_add_task(s)
+            self.rawserver.add_task(s)
             if self.finflag.isSet():        # mode started when already finished
                 def r(self = self):
                     self.rerequest.announce(3)  # so after kicking everyone off, reannounce
-                self.rawserver.external_add_task(r)
+                self.rawserver.add_task(r)
 
     def am_I_finished(self):
         return self.finflag.isSet()
 
     def get_transfer_stats(self):
-        return self.upmeasure.get_total(), self.downmeasure.get_total()
\ No newline at end of file
+        return self.upmeasure.get_total(), self.downmeasure.get_total()
