@@ -117,8 +117,6 @@ class InvokeEvent(wxPyEvent):
         self.args = args
         self.kwargs = kwargs
 
-def pr(event):
-    print 'augh!'
 
 
 class DownloadInfoFrame:
@@ -176,15 +174,17 @@ class DownloadInfoFrame:
 
             wxInitAllImageHandlers()
             self.basepath = self.configfile.getIconDir()
-            self.statusIcons={
-                'startup':wxIcon(os.path.join(self.basepath,'white.ico'), wxBITMAP_TYPE_ICO),
-                'disconnected':wxIcon(os.path.join(self.basepath,'black.ico'), wxBITMAP_TYPE_ICO),
-                'noconnections':wxIcon(os.path.join(self.basepath,'red.ico'), wxBITMAP_TYPE_ICO),
-                'nocompletes':wxIcon(os.path.join(self.basepath,'blue.ico'), wxBITMAP_TYPE_ICO),
-                'noincoming':wxIcon(os.path.join(self.basepath,'yellow.ico'), wxBITMAP_TYPE_ICO),
-                'allgood':wxIcon(os.path.join(self.basepath,'green.ico'), wxBITMAP_TYPE_ICO)
+            self.icon = wxIcon(os.path.join(self.basepath,'icon_bt.ico'), wxBITMAP_TYPE_ICO)
+            self.finicon = wxIcon(os.path.join(self.basepath,'icon_done.ico'), wxBITMAP_TYPE_ICO)
+            self.statusIconFiles={
+                'startup':os.path.join(self.basepath,'white.ico'),
+                'disconnected':os.path.join(self.basepath,'black.ico'),
+                'noconnections':os.path.join(self.basepath,'red.ico'),
+                'nocompletes':os.path.join(self.basepath,'blue.ico'),
+                'noincoming':os.path.join(self.basepath,'yellow.ico'),
+                'allgood':os.path.join(self.basepath,'green.ico'),
                 }
-
+            self.statusIcons={}
             self.filestatusIcons = wxImageList(16, 16)
             self.filestatusIcons.Add(wxBitmap(os.path.join(self.basepath,'black1.ico'),wxBITMAP_TYPE_ICO))
             self.filestatusIcons.Add(wxBitmap(os.path.join(self.basepath,'yellow1.ico'), wxBITMAP_TYPE_ICO))
@@ -192,13 +192,13 @@ class DownloadInfoFrame:
 
             self.allocbuttonBitmap = wxBitmap(os.path.join(self.basepath,'alloc.gif'), wxBITMAP_TYPE_GIF)
 
-            if (sys.platform == 'win32'):
-                self.icon = wxIcon(os.path.join(self.basepath,'icon_bt.ico'), wxBITMAP_TYPE_ICO)
             self.starttime = clock()
 
             self.frame = frame
-            if (sys.platform == 'win32'):
+            try:
                 self.frame.SetIcon(self.icon)
+            except:
+                pass
 
             panel = wxPanel(frame, -1)
             self.bgcolor = panel.GetBackgroundColour()
@@ -272,7 +272,6 @@ class DownloadInfoFrame:
 
             self.gauge = wxGauge(panel, -1, range = 1000, style = wxGA_SMOOTH)
             colSizer.Add(self.gauge, 0, wxEXPAND)
-#        self.gauge.SetForegroundColour(wxSystemSettings_GetColour(wxSYS_COLOUR_3DSHADOW))
 
             timeSizer = wxFlexGridSizer(cols = 2)
             timeSizer.Add(StaticText('Time elapsed / estimated : '))
@@ -340,11 +339,9 @@ class DownloadInfoFrame:
 
             cancelSizer=wxGridSizer(cols = 2, hgap = 40)
             self.pauseButton = wxButton(panel, -1, 'Pause')
-#        self.pauseButton.SetFont(self.default_font)
             cancelSizer.Add(self.pauseButton, 0, wxALIGN_CENTER)
 
             self.cancelButton = wxButton(panel, -1, 'Cancel')
-#        self.cancelButton.SetFont(self.default_font)
             cancelSizer.Add(self.cancelButton, 0, wxALIGN_CENTER)
             colSizer.Add(cancelSizer, 0, wxALIGN_CENTER)
 
@@ -409,7 +406,6 @@ class DownloadInfoFrame:
             colSizer.Add(self.unlimitedLabel, 0, wxALIGN_CENTER)
 
             self.priorityIDs = [wxNewId(),wxNewId(),wxNewId(),wxNewId()]
-#            self.prioritycolors = [wxLIGHT_GREY, wxRED, wxBLACK, wxBLUE]
             self.prioritycolors = [ wxColour(160,160,160),
                                     wxColour(255,64,0),
                                     wxColour(0,0,0),
@@ -486,16 +482,33 @@ class DownloadInfoFrame:
                 wxPostEvent(self.frame, InvokeEvent(func, args, kwargs))
 
 
+    def getStatusIcon(self, name, bitmap=False):
+        if self.statusIcons.has_key(name):
+            i = self.statusIcons[name]
+            if type(i)  == type(self.icon) and not bitmap:
+                return i
+        if bitmap:
+            i = wxBitmap(self.statusIconFiles[name], wxBITMAP_TYPE_ICO)
+        else:
+            i = wxIcon(self.statusIconFiles[name], wxBITMAP_TYPE_ICO)
+        self.statusIcons[name] = i
+        return i
+
+
     def setStatusIcon(self, name):
-        if name != self.statusIconValue:
-            self.statusIconValue = name
-            statidata = wxMemoryDC()
-            statidata.SelectObject(self.statusIcon)
-            statidata.BeginDrawing()
-            statidata.DrawIcon(self.statusIcons[name],0,0)
-            statidata.EndDrawing()
-            statidata.SelectObject(wxNullBitmap)
-            self.statusIconPtr.Refresh()
+        if name == self.statusIconValue:
+            return
+        self.statusIconValue = name
+        statidata = wxMemoryDC()
+        statidata.SelectObject(self.statusIcon)
+        statidata.BeginDrawing()
+        try:
+            statidata.DrawIcon(self.getStatusIcon(name),0,0)
+        except:
+            statidata.DrawBitmap(self.getStatusIcon(name,True),0,0,True)
+        statidata.EndDrawing()
+        statidata.SelectObject(wxNullBitmap)
+        self.statusIconPtr.Refresh()
 
 
     def createStatusIcon(self, name):
@@ -505,7 +518,10 @@ class DownloadInfoFrame:
         bbdata.SetPen(wxTRANSPARENT_PEN)
         bbdata.SetBrush(wxBrush(self.bgcolor,wxSOLID))
         bbdata.DrawRectangle(0,0,32,32)
-        bbdata.DrawIcon(self.statusIcons[name],0,0)
+        try:
+            bbdata.DrawIcon(self.getStatusIcon(name),0,0)
+        except:
+            bbdata.DrawBitmap(self.getStatusIcon(name,True),0,0,True)
         return iconbuffer
 
 
@@ -530,26 +546,17 @@ class DownloadInfoFrame:
     def onIconify(self, evt):
         try:
             if self.configfileargs['win32_taskbar_icon']:
-                self.frame.tbicon.SetIcon(self.icon, "BitTorrent")
+                if self.fin:
+                    self.frame.tbicon.SetIcon(self.finicon, "BitTorrent")
+                else:
+                    self.frame.tbicon.SetIcon(self.icon, "BitTorrent")
                 self.frame.Hide()
                 self.taskbaricon = True
             else:
-                EVT_ICONIZE(self.frame, self.onIconifyDummy)
-                if self.iconized:
-                    self.frame.Iconize(False)
-                    self.iconized = False
-                else:
-                    self.frame.Iconize(True)
-                    self.iconized = True
-                EVT_ICONIZE(self.frame, self.onIconify)
-                # rant here -- why in god's name can't a function called by an event
-                # trigger the event without calling itself?
-                # self.frame.Iconize(not self.frame.IsIconized()) should do this job...
+                return
         except:
             self.exception()
 
-    def onIconifyDummy(self, evt):
-        return
 
     def onTaskBarActivate(self, evt):
         try:
@@ -560,6 +567,8 @@ class DownloadInfoFrame:
                 self.frame.Raise()
             self.frame.tbicon.RemoveIcon()
             self.taskbaricon = False
+        except wxPyDeadObjectError:
+            pass
         except:
             self.exception()
 
@@ -717,8 +726,10 @@ class DownloadInfoFrame:
 
             self.aboutBox = wxFrame(None, -1, 'About BitTorrent', size = (1,1),
                             style = wxDEFAULT_FRAME_STYLE|wxFULL_REPAINT_ON_RESIZE)
-            if (sys.platform == 'win32'):
+            try:
                 self.aboutBox.SetIcon(self.icon)
+            except:
+                pass
 
             panel = wxPanel(self.aboutBox, -1)
 
@@ -767,7 +778,6 @@ class DownloadInfoFrame:
             colSizer.Add (babble6)
 
             okButton = wxButton(panel, -1, 'Ok')
-#        okButton.SetFont(self.default_font)
             colSizer.Add(okButton, 0, wxALIGN_RIGHT)
             colSizer.AddGrowableCol(0)
 
@@ -797,11 +807,15 @@ class DownloadInfoFrame:
                 frame.aboutBox.Close ()
             EVT_BUTTON(self.aboutBox, okButton.GetId(), closeAbout)
             def kill(self, frame = self):
+                try:
+                    frame.RemoveIcon()
+                except:
+                    pass
                 frame.aboutBox.Destroy()
                 frame.aboutBox = None
             EVT_CLOSE(self.aboutBox, kill)
 
-            self.aboutBox.Show ()
+            self.aboutBox.Show()
             border.Fit(panel)
             self.aboutBox.Fit()
         except:
@@ -835,8 +849,10 @@ class DownloadInfoFrame:
 
             self.detailBox = wxFrame(None, -1, 'Torrent Details ', size = wxSize(405,230),
                             style = wxDEFAULT_FRAME_STYLE|wxFULL_REPAINT_ON_RESIZE)
-            if (sys.platform == 'win32'):
+            try:
                 self.detailBox.SetIcon(self.icon)
+            except:
+                pass
 
             panel = wxPanel(self.detailBox, -1, size = wxSize (400,220))
 
@@ -897,12 +913,9 @@ class DownloadInfoFrame:
                 fileList.InsertColumn(1, "", format=wxLIST_FORMAT_RIGHT, width=55)
                 fileList.InsertColumn(2, "")
 
-#                self.fileListItems = []
                 for i in range(len(info['files'])):
                     x = wxListItem()
-#                x.SetFont(self.default_font)
                     fileList.InsertItem(x)
-#                    self.fileListItems.append(x)
 
                 x = 0
                 for file in info['files']:
@@ -952,12 +965,10 @@ class DownloadInfoFrame:
                 for tier in range(len(announce_list)):
                     for t in range(len(announce_list[tier])):
                         i = wxListItem()
-#                    i.SetFont(self.default_font)
                         trackerList.InsertItem(i)
                 if announce is not None:
                     for l in [1,2]:
                         i = wxListItem()
-#                    i.SetFont(self.default_font)
                         trackerList.InsertItem(i)
 
                 x = 0
@@ -1000,7 +1011,6 @@ class DownloadInfoFrame:
             colSizer.Add (detailSizer, 1, wxEXPAND)
 
             okButton = wxButton(panel, -1, 'Ok')
-#        okButton.SetFont(self.default_font)
             colSizer.Add(okButton, 0, wxALIGN_RIGHT)
             colSizer.AddGrowableCol(0)
 
@@ -1016,7 +1026,6 @@ class DownloadInfoFrame:
 
             if fileListID:
                 def onRightClick(evt, self = self):
-#                    s = evt.GetIndex()
                     s = []
                     i = -1
                     while True:
@@ -1067,6 +1076,10 @@ class DownloadInfoFrame:
                 self.detailBox.Close ()
             EVT_BUTTON(self.detailBox, okButton.GetId(), closeDetail)
             def kill(evt, self = self):
+                try:
+                    self.detailBox.RemoveIcon()
+                except:
+                    pass
                 self.detailBox.Destroy()
                 self.detailBox = None
                 self.fileList = None
@@ -1094,14 +1107,16 @@ class DownloadInfoFrame:
         try:
             if (self.creditsBox is not None):
                 try:
-                    self.creditsBox.Close ()
+                    self.creditsBox.Close()
                 except wxPyDeadObjectError, e:
                     self.creditsBox = None
 
             self.creditsBox = wxFrame(None, -1, 'Credits', size = (1,1),
                             style = wxDEFAULT_FRAME_STYLE|wxFULL_REPAINT_ON_RESIZE)
-            if (sys.platform == 'win32'):
+            try:
                 self.creditsBox.SetIcon(self.icon)
+            except:
+                pass
 
             panel = wxPanel(self.creditsBox, -1)        
 
@@ -1161,7 +1176,6 @@ class DownloadInfoFrame:
               'Micah Anderson'))
             colSizer.Add (creditSizer, flag = wxALIGN_CENTER_HORIZONTAL)
             okButton = wxButton(panel, -1, 'Ok')
-#        okButton.SetFont(self.default_font)
             colSizer.Add(okButton, 0, wxALIGN_RIGHT)
             colSizer.AddGrowableCol(0)
 
@@ -1171,9 +1185,13 @@ class DownloadInfoFrame:
             panel.SetAutoLayout(True)
 
             def closeCredits(self, frame = self):
-                frame.creditsBox.Close ()
+                frame.creditsBox.Close()
             EVT_BUTTON(self.creditsBox, okButton.GetId(), closeCredits)
             def kill(self, frame = self):
+                try:
+                    frame.creditsBox.RemoveIcon()
+                except:
+                    pass
                 frame.creditsBox.Destroy()
                 frame.creditsBox = None
             EVT_CLOSE(self.creditsBox, kill)
@@ -1189,14 +1207,16 @@ class DownloadInfoFrame:
         try:
             if (self.statusIconHelpBox is not None):
                 try:
-                    self.statusIconHelpBox.Close ()
+                    self.statusIconHelpBox.Close()
                 except wxPyDeadObjectError, e:
                     self.statusIconHelpBox = None
 
             self.statusIconHelpBox = wxFrame(None, -1, 'Help with the BitTorrent Status Light', size = (1,1),
                             style = wxDEFAULT_FRAME_STYLE|wxFULL_REPAINT_ON_RESIZE)
-            if (sys.platform == 'win32'):
+            try:
                 self.statusIconHelpBox.SetIcon(self.icon)
+            except:
+                pass
 
             panel = wxPanel(self.statusIconHelpBox, -1)
 
@@ -1263,7 +1283,6 @@ class DownloadInfoFrame:
             colsizer2.Add(colspacer)
 
             okButton = wxButton(panel, -1, 'Ok')
-#        okButton.SetFont(self.default_font)
             colsizer2.Add(okButton, 0, wxALIGN_CENTER)
             fullsizer.Add(colsizer2, 0, wxALIGN_CENTER)
 
@@ -1275,7 +1294,7 @@ class DownloadInfoFrame:
 
 
             def closeHelp(self, frame = self):
-                frame.statusIconHelpBox.Close ()
+                frame.statusIconHelpBox.Close()
             EVT_BUTTON(self.statusIconHelpBox, okButton.GetId(), closeHelp)
 
             self.statusIconHelpBox.Show ()
@@ -1304,8 +1323,10 @@ class DownloadInfoFrame:
 
             self.advBox = wxFrame(None, -1, 'BitTorrent Advanced', size = wxSize(200,200),
                             style = wxDEFAULT_FRAME_STYLE|wxFULL_REPAINT_ON_RESIZE)
-            if (sys.platform == 'win32'):
+            try:
                 self.advBox.SetIcon(self.icon)
+            except:
+                pass
 
             panel = wxPanel(self.advBox, -1, size = wxSize (200,200))
 
@@ -1359,25 +1380,20 @@ class DownloadInfoFrame:
             buttonSizer = wxFlexGridSizer (cols = 5, hgap = 20)
 
             reannounceButton = wxButton(panel, -1, 'Manual Announce')
-#        reannounceButton.SetFont(self.default_font)
             buttonSizer.Add (reannounceButton)
 
             extannounceButton = wxButton(panel, -1, 'External Announce')
-#        extannounceButton.SetFont(self.default_font)
             buttonSizer.Add (extannounceButton)
 
             bgallocButton = wxButton(panel, -1, 'Finish Allocation')
-#        bgallocButton.SetFont(self.default_font)
             buttonSizer.Add (bgallocButton)
 
             buttonSizer.Add(StaticText(''))
 
             okButton = wxButton(panel, -1, 'Ok')
-#        okButton.SetFont(self.default_font)
             buttonSizer.Add (okButton)
 
             colSizer.Add (buttonSizer, 0, wxALIGN_CENTER)
-#        colSizer.SetMinSize ((578,350))
             colSizer.AddGrowableCol(0)
             colSizer.AddGrowableRow(1)
 
@@ -1416,8 +1432,10 @@ class DownloadInfoFrame:
 
                 frame.advextannouncebox = wxFrame(None, -1, 'External Announce', size = (1,1),
                             style = wxDEFAULT_FRAME_STYLE|wxFULL_REPAINT_ON_RESIZE)
-                if (sys.platform == 'win32'):
+                try:
                     frame.advextannouncebox.SetIcon(frame.icon)
+                except:
+                    pass
 
                 panel = wxPanel(frame.advextannouncebox, -1)
 
@@ -1435,11 +1453,9 @@ class DownloadInfoFrame:
                 buttonSizer = wxFlexGridSizer (cols = 2, hgap = 10)
 
                 okButton = wxButton(panel, -1, 'OK')
-#            okButton.SetFont(frame.default_font)
                 buttonSizer.Add (okButton)
 
                 cancelButton = wxButton(panel, -1, 'Cancel')
-#            cancelButton.SetFont(frame.default_font)
                 buttonSizer.Add (cancelButton)
 
                 fullsizer.Add (buttonSizer, 0, wxALIGN_CENTER)
@@ -1464,11 +1480,6 @@ class DownloadInfoFrame:
                     frame.advextannouncebox.Close()
                 EVT_BUTTON(frame.advextannouncebox, cancelButton.GetId(), cancel)
 
-                def kill(self, frame = frame):
-                    frame.advextannouncebox.Destroy()
-                    frame.advextannouncebox = None
-                EVT_CLOSE(frame.advextannouncebox, kill)
-
                 frame.advextannouncebox.Show ()
                 fullsizer.Fit(panel)
                 frame.advextannouncebox.Fit()
@@ -1481,15 +1492,19 @@ class DownloadInfoFrame:
             EVT_BUTTON(self.advBox, bgallocButton.GetId(), bgalloc)
 
             def closeAdv(evt, self = self):
-                self.advBox.Close ()
+                self.advBox.Close()
             def killAdv(evt, self = self):
+                try:
+                    self.advBox.RemoveIcon()
+                except:
+                    pass
                 self.onDownRateSpinner()
                 self.dow.spewflag.clear()
                 self.advBox.Destroy()
                 self.advBox = None
                 if (self.advextannouncebox is not None):
                     try:
-                        self.advextannouncebox.Close ()
+                        self.advextannouncebox.Close()
                     except wxPyDeadObjectError, e:
                         pass
                     self.advextannouncebox = None
@@ -1549,8 +1564,7 @@ class DownloadInfoFrame:
         if activity is not None:
             self.activity = activity
         self.gui_fractiondone = fractionDone
-        if not self.ispaused:
-            self.invokeLater(self.onUpdateStatus,
+        self.invokeLater(self.onUpdateStatus,
                  [dpflag, timeEst, downRate, upRate, statistics, spew, sizeDone])
 
     def onUpdateStatus(self, dpflag, timeEst, downRate, upRate,
@@ -1571,7 +1585,7 @@ class DownloadInfoFrame:
             self.firstupdate = False
             if self.advBox:
                 self.dow.spewflag.set()
-        if statistics is None:
+        if self.ispaused or statistics is None:
             self.setStatusIcon('startup')
         elif statistics.numPeers + statistics.numSeeds + statistics.numOldSeeds == 0:
             if statistics.last_failed:
@@ -1626,17 +1640,17 @@ class DownloadInfoFrame:
                     self.frame.SetTitle('%.1f%% %s - BitTorrent %s' % (float(gaugelevel)/10, self.filename, version))
             else:
                 self.frame.SetTitle('%.0f%% %s - BitTorrent %s' % (float(gaugelevel)/10, self.filename, version))
-        if timeEst is not None:
-            self.timeText.SetLabel(hours(clock() - self.starttime) + ' / ' + hours(timeEst))
+        if self.ispaused:
+            self.timeText.SetLabel(hours(clock() - self.starttime) + ' /')
+        elif timeEst is None:
+            self.timeText.SetLabel(hours(clock() - self.starttime) + ' / ' + self.activity)
         else:
-            if self.fin:
-                self.timeText.SetLabel(self.activity)
-            else:
-                self.timeText.SetLabel(hours(clock() - self.starttime) + ' / ' + self.activity)
-        if downRate is not None:
-            self.downRateText.SetLabel('%.0f kB/s' % (float(downRate) / 1000))
-        if upRate is not None:
-            self.upRateText.SetLabel('%.0f kB/s' % (float(upRate) / 1000))
+            self.timeText.SetLabel(hours(clock() - self.starttime) + ' / ' + hours(timeEst))
+        if not self.ispaused:
+            if downRate is not None:
+                self.downRateText.SetLabel('%.0f kB/s' % (float(downRate) / 1000))
+            if upRate is not None:
+                self.upRateText.SetLabel('%.0f kB/s' % (float(upRate) / 1000))
         if self.taskbaricon:
             icontext='BitTorrent '
             if self.gui_fractiondone is not None and not self.fin:
@@ -1650,7 +1664,10 @@ class DownloadInfoFrame:
                 icontext=icontext+' d:%.0f kB/s' % (float(downRate) / 1000)
             icontext+=' %s' % self.filename
             try:
-                self.frame.tbicon.SetIcon(self.icon,icontext)
+                if self.fin:
+                    self.frame.tbicon.SetIcon(self.finicon,icontext)
+                else:
+                    self.frame.tbicon.SetIcon(self.icon,icontext)
             except:
                 pass
         if statistics is not None:
@@ -1725,7 +1742,6 @@ class DownloadInfoFrame:
                     kickbanlen = 0
                 for x in range(spewlen-spewList.GetItemCount()):
                     i = wxListItem()
-#                   i.SetFont(self.default_font)
                     spewList.InsertItem(i)
                 for x in range(spewlen,spewList.GetItemCount()):
                     spewList.DeleteItem(len(spew)+1)
@@ -1931,11 +1947,12 @@ class DownloadInfoFrame:
         self.cancelButton.SetLabel('Finish')
         self.gauge.SetValue(0)
         self.frame.SetTitle('%s - Upload - BitTorrent %s' % (self.filename, version))
-        if (sys.platform == 'win32'):
-            self.icon = wxIcon(os.path.join(self.basepath,'icon_done.ico'), wxBITMAP_TYPE_ICO)
-            self.frame.SetIcon(self.icon)
+        try:
+            self.frame.SetIcon(self.finicon)
+        except:
+            pass
         if self.taskbaricon:
-            self.frame.tbicon.SetIcon(self.icon, "BitTorrent - Finished")
+            self.frame.tbicon.SetIcon(self.finicon, "BitTorrent - Finished")
         self.downRateText.SetLabel('')
 
     def onFailEvent(self):
@@ -1952,7 +1969,7 @@ class DownloadInfoFrame:
             self.errorText.SetLabel(errormsg)
             self.lastError = clock()
         else:
-            self.errorText.SetLabel(strftime('ERROR (%I:%M %p) -\n') + errormsg)
+            self.errorText.SetLabel(strftime('ERROR (%x %X) -\n') + errormsg)
             self.lastError = clock()
 
 
@@ -2060,8 +2077,6 @@ class DownloadInfoFrame:
                 self.pauseButton.SetLabel('Resume')
                 self.downRateText.SetLabel(' ')
                 self.upRateText.SetLabel(' ')
-                self.seedStatusText.SetLabel(' ')
-                self.peerStatusText.SetLabel(' ')
                 self.setStatusIcon('startup')
 
     def done(self, event):
@@ -2070,8 +2085,10 @@ class DownloadInfoFrame:
         self.shuttingdown = True
         if self.taskbaricon:
             self.frame.tbicon.RemoveIcon()
-        if self.ispaused:
-            self.dow.Unpause()
+            try:
+                self.frame.tbicon.Destroy()
+            except:
+                pass
         if (self.detailBox is not None):
             try:
                 self.detailBox.Close ()
@@ -2099,7 +2116,13 @@ class DownloadInfoFrame:
             except wxPyDeadObjectError, e:
                 self.statusIconHelpBox = None
         self.configfile.Close()
+        try:
+            self.frame.RemoveIcon()
+        except:
+            pass
         self.frame.Destroy()
+        self.icon.Destroy()
+        self.finicon.Destroy()
 
     def exception(self):
         data = StringIO()
